from wx import Image, BITMAP_TYPE_ANY
import os

SERVER = "199.167.193.68"
#SERVER = "localhost"

GRID_SIZE = 15

def building_at(buildings,x,y):
    for building in buildings:
        for square in building.squares():
            if square == (x,y):
                return building
            
    return False

class Building(object):
    w=1
    h=1
    cost=1
    help_text = "Cost: %d\nSize: %dx%d"%(cost,w,h)
    max_targets = 0
    hostile=False
    production = 0
    buildpoints = 0
    charge = 0
    needs_charge = False
    target_radius = 1
    def __init__(self,x,y,orientation = False):
        self.x = int(x)
        self.y = int(y)
        self.orientation = orientation
        self.targets = []
        self.damage = []

    def target(self,target):
        if (target[0] == 2 or not (target[1], target[2]) in self.squares()) and self.hostile+1 == target[0]:
            self.targets.append(target)
            print "Added target: ",target
        while len(self.targets) > self.max_targets:
            self.targets.pop(0)
            
    def target_squares(self):
        for g,x,y in self.targets:
            print x,y,range(x-self.target_radius+1,x+self.target_radius), range(y-self.target_radius+1,y+self.target_radius)
            for dx in xrange(x-self.target_radius+1,x+self.target_radius):
                for dy in xrange(y-self.target_radius+1,y+self.target_radius):
                    if 0 <= dx < GRID_SIZE and 0 <= dy < GRID_SIZE:
                        yield (dx,dy)
            
        
    def squares(self):
        for x in range(self.x,self.x+self.width):
            for y in range(self.y,self.y+self.height):
                yield (x,y)
        
    def width(self):
        if self.orientation: return self.h
        return self.w
    width = property(width)
    
    def height(self):
        if self.orientation: return self.w
        return self.h
    height = property(height)
    
    def hit(self,x,y):
        if not (x,y) in self.damage and (x,y) in self.squares():
            self.damage.append((x,y))
            return True
        return False
    
    def destroyed(self):
        return len(self.damage) == self.h*self.w


class Laboratory(Building):
    verbose_name = "Laboratory"
    image = Image(os.path.join("resources","tiles","laboratory.png"), type=BITMAP_TYPE_ANY)
    w = 1
    h = 1
    cost = 1
    production = 1
    help_text = "Cost: %d\nSize: %dx%d\nIncreases Production By 1"%(cost,w,h)

class NaniteFactory(Building):
    verbose_name = "Nanite Factory"
    image = Image(os.path.join("resources","tiles","nanite.png"), type=BITMAP_TYPE_ANY)
    w = 1
    h = 1
    cost = 2
    buildpoints = 1
    
    help_text = "Cost: %d\nSize: %dx%d\nIncreases the number of building that can be built per turn"%(cost,w,h)
    
class ArcReactor(Building):
    verbose_name = "Arc Reactor"
    image = Image(os.path.join("resources","tiles","reactor.png"), type=BITMAP_TYPE_ANY)
    w = 2
    h = 3
    cost = 6
    max_targets = 5
    
    help_text = "Cost: %d\nSize: %dx%d\nIs used to provide power to buildings"%(cost,w,h)
    
class LightMissile(Building):
    verbose_name = "Light Missile"
    image = Image(os.path.join("resources","tiles","light_missile.png"), type=BITMAP_TYPE_ANY)
    w = 1
    h = 1
    cost = 2
    max_targets = 1
    hostile = True
    
    help_text = "Cost: %d\nSize: %dx%d\nCan Launch a single small strike"%(cost,w,h)
    
class HeavyMissile(Building):
    verbose_name = "Heavy Missile"
    image = Image(os.path.join("resources","tiles","heavy_missile.png"), type=BITMAP_TYPE_ANY)
    w = 2
    h = 1
    cost = 4
    max_targets = 1
    target_radius = 2
    hostile = True
    
    help_text = "Cost: %d\nSize: %dx%d\nCan Launch a single large strike"%(cost,w,h)
    
class Deathray(Building):
    verbose_name = "Deathray"
    image = Image(os.path.join("resources","tiles","deathray.png"), type=BITMAP_TYPE_ANY)
    w = 2
    h = 2
    cost = 8
    hostile = True
    max_targets = 2
    target_radius = 3
    needs_charge = True
    
    help_text = "Cost: %d\nSize: %dx%d\n When powered can launch 2 really heavy strikes"%(cost,w,h)


buildings = [Laboratory,NaniteFactory,ArcReactor,LightMissile,HeavyMissile,Deathray]