from __future__ import with_statement
import Pyro4

import engine

Pyro4.config.COMPRESSION = True
Pyro4.config.HOST = engine.SERVER
Pyro4.config.SERVERTYPE = "multiplex"

class GameServer(object):
    def __init__(self):
        self.games={}        # registered channels { channel --> (nick, client callback) list }
        self.gamecount = 0
        #self.players={}      #
        self.waiting = []
        
    #def join_server(self, name):
    #    if not name:
    #        raise ValueError("invalid name")
    #    if name in self.players:
    #        try:
    #            if callback.ping():
    #                raise ValueError('this name is already in use')
    #        except Pyro4.errors.ConnectionClosedError:
    #            del self.players[name]
    #            print "CLEANED UP %s"%name
    #    
    #    callback._pyroOneway.add('next_move')    # don't wait for results for this method
    #    self.players[name] = callback
    #    
    #    print "CONNECTED %s"%name
    #    
    #    for game in self.games.values():
    #        if game.has_player(name) and game.waiting_for(name):
    #            callback.next_move(game)
    #            
    #    return True

    def join_game(self,name):
        if self.waiting and self.waiting.count(name) < len(self.waiting):
            while name == self.waiting[0]:
                self.waiting.append(self.waiting.pop(0))
            new_game = Game(self,self.gamecount,name,self.waiting.pop(0))
            self.gamecount += 1
            self.games[new_game.id] = new_game
            self._pyroDaemon.register(new_game)
            print "GAME STARTED BETWEEN %s and %s"%tuple(new_game.players.keys())
            return new_game
        else:
            self.waiting.append(name)
            return False
    
    def get_all_games_waiting(self,player):
        return [game for game in self.games.values() if (game.has_player(player) and game.waiting_for(player))]
        
    def get_all_games(self,player):
        return [game for game in self.games.values() if (game.has_player(player))]
        
    #def get_all_players(self):
    #    return self.players
    
    def ping(self): return True
                    
class Game(object):
    def __init__(self,server,id,player1,player2):
        self.server = server
        self.id = id
        self.players = {}
        self.players[player1] = Player(player1)
        self.players[player2] = Player(player2)
        
    def GetId(self):
        return self.id
    
    def game_over(self,player):
        for nplayer in self.players.values():
            if (not nplayer.first_turn) and len(nplayer.buildings) == 0 and len(self.players[self.other_player(nplayer.name)].buildings) != 0:
                self.players[player].finished = True
                if self.players[player].finished and self.players[self.other_player(player)].finished:
                    del self.server.games[self.id]
                    print "Cleaned up game %d"%self.id
                return self.players[self.other_player(nplayer.name)].name
        return False
        
    def waiting_for(self,player):
        return not self.players[player].had_turn
        
    def has_player(self,player):
        return player in self.players

    def get_buildings(self,player):
        return self.players[player].buildings
        
    def get_destroyed(self,player):
        return self.players[player].destroyed
        
    def get_enemy_destroyed(self,player):
        return self.players[self.other_player(player)].destroyed
        
    def get_enemy_hits_and_misses(player):
        player = self.other_player(player)
        return self.players[player].new_hits,self.players[player].new_misses,self.players[player].hits,self.players[player].misses
        
    #def get_buildpoints(self,player):
    #    return self.players[player].buildpoints()
        
    #def get_production(self,player):
    #    return self.players[player].production()
        
    def get_hits_and_misses(self,player):
        return self.players[player].new_hits,self.players[player].new_misses,self.players[player].hits,self.players[player].misses
        
    def make_move(self,player,buildings,new_buildings):
        player = self.players[player]
        player.buildings = buildings
        accum_build_points = 0
        build_points = sum(building.buildpoints for building in player.buildings) + 1
        accum_cost = 0
        production = sum(building.production for building in player.buildings) + 1
        
        for building in new_buildings:
            if  accum_build_points+1 <= build_points and accum_cost+building.cost <= production:
                player.buildings.append(building)
            else:
                print "%s Is attempting to cheat skipped building"%player.name
        player.had_turn = True
        if self.players[self.other_player(player.name)].had_turn:
            self.do_turn()
            return True
        return False
        
    def do_turn(self):
        print "TURN BETWEEN %s and %s"%tuple(self.players.keys())
        for player in self.players.values():
            for building in player.buildings:
                building.charge = 0
                
        for player in self.players.values():
            player.hits |= player.new_hits
            player.new_hits = set()
            player.misses |= player.new_misses
            player.new_misses = set()
            for building in player.buildings:
                if not building.hostile:
                    for target in building.targets:
                        for building_2 in player.buildings:
                            if target[1:] in building_2.squares():
                                building_2.charge+=1
                                
        for player in self.players.values():
            player.new_shots = []
            for building in player.buildings:
                if building.hostile:
                    for target in building.target_squares():
                        player.new_shots.append(target)
                    building.targets = []
        
        for player in self.players.values():
            other = self.players[self.other_player(player.name)]
            for shot in player.new_shots:
                for building in other.buildings:
                    if building.hit(*shot):
                        player.new_hits.add(shot)
                    else:
                        player.new_misses.add(shot)
                    if building.destroyed():
                        other.buildings.remove(building)
                        other.destroyed.append(building)
            player.new_misses -= player.new_hits
            player.new_misses -= player.hits
                    
            player.had_turn = False
            player.first_turn = False
        
    def other_player(self,player):
        for key in self.players.keys():
            if key != player:
                return key

class Player(object):
    def __init__(self,name):
        self.name = name
        self.had_turn = False
        self.buildings = []
        self.destroyed = []
        self.misses=set()
        self.hits=set()
        self.new_hits = set()
        self.new_misses = set()
        self.new_shots = []
        self.finished= False
        self.first_turn=True
        
    def buildpoints(self):
        return sum(building.buildpoints for building in self.buildings)
        
    def production(self):
        return sum(building.production for building in self.buildings)
        

with Pyro4.core.Daemon() as daemon:
    with Pyro4.naming.locateNS() as ns:
        uri=daemon.register(GameServer())
        ns.register("deathray.gameserver",uri)
    # enter the service loop.
    print('Game server started')
    daemon.requestLoop()
